package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.annotation.JSONField;
import com.github.pagehelper.PageHelper;
import com.tzld.piaoquan.risk.control.config.LoginUserContext;
import com.tzld.piaoquan.risk.control.dao.mapper.UserBaseMapper;
import com.tzld.piaoquan.risk.control.model.po.UserBase;
import com.tzld.piaoquan.risk.control.model.po.UserBaseExample;
import com.tzld.piaoquan.risk.control.model.qywx.QwCallBackWrapperData;
import com.tzld.piaoquan.risk.control.service.qywx.Constant;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class QywxUserDataService {

    @Autowired
    private UserBaseMapper userBaseMapper;

    @Data
    @NoArgsConstructor
    private static class UserData {
        // 企业信息
        @JSONField(name = "Corpid")
        private Long corpId;

        @JSONField(name = "corp_id")
        private Long corpId2;

        @JSONField(name = "Corp_name")
        private String corpName;

        @JSONField(name = "Corp_full_name")
        private String corpFullName;

        @JSONField(name = "Corp_logo")
        private String corpLogo;

        @JSONField(name = "Corp_desc")
        private String corpDesc;

        @JSONField(name = "scorp_id")
        private String sCorpId;

        // 用户基础信息
        @JSONField(name = "unionid")
        private String unionId;

        @JSONField(name = "Admin_vid")
        private Long adminVid;

        @JSONField(name = "acctid")
        private String accountId;

        @JSONField(name = "english_name")
        private String englishName;

        @JSONField(name = "Ownername")
        private String ownerName;

        @JSONField(name = "nickname")
        private String nickName;

        @JSONField(name = "create_time")
        private Long createTime1;

        @JSONField(name = "Create_time")
        private Long createTime2;

        @JSONField(name = "sex")
        private Integer gender;

        @JSONField(name = "mobile")
        private String mobile;

        @JSONField(name = "avatar")
        private String avatarUrl;

        @JSONField(name = "realname")
        private String realName;

        @JSONField(name = "Vid")
        private Long vid;

        @JSONField(name = "Mail")
        private String email;

        @JSONField(name = "user_id")
        private Long userId;

        @JSONField(name = "position")
        private String jobPosition;
    }

    public void loginSuccess(String originalJson) {
        // 1. 解析外层数据
        QwCallBackWrapperData wrapperData = JSON.parseObject(originalJson, QwCallBackWrapperData.class);
        if (StringUtils.isEmpty(wrapperData.getJson())) {
            log.warn("Empty inner json data, uuid: {}", wrapperData.getUuid());
            return;
        }

        UserData userData = JSON.parseObject(wrapperData.getJson(), UserData.class);
        processBaseInfo(wrapperData, userData);
    }

    private void processBaseInfo(QwCallBackWrapperData envelop, UserData data) {
        UserBase userBase = new UserBase();
        userBase.setUuid(envelop.getUuid());
        userBase.setVid(data.vid.toString());
        userBase.setMobile(data.mobile);
        userBase.setRealname(data.realName);
        userBase.setEnglishName(data.englishName);
        userBase.setAvatar(data.avatarUrl);
        userBase.setLoginStatus(envelop.getType() == Constant.LOGIN_SUCCESS ? 1 : 0);
        userBase.setCorpId(data.corpId);
        userBase.setCorpName(data.corpName);
        userBase.setUnionid(data.unionId);
        userBase.setPosition(data.jobPosition);
        // 时间处理（优先使用Create_time）
        Long createTime = data.createTime2 != null && data.createTime2 > 0 ? data.createTime2 : data.createTime1;
        if (createTime != null && createTime > 0) {
            userBase.setCreateTime(new Date(createTime * 1000));
        }

        // 2. 根据 vid 删除所有旧记录
        UserBaseExample deleteExample = new UserBaseExample();
        deleteExample.createCriteria().andVidEqualTo(data.vid.toString());
        int deletedCount = userBaseMapper.deleteByExample(deleteExample);
        log.info("Deleted {} old records for vid: {}", deletedCount, data.vid);

        // 查询是否已存在
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andUuidEqualTo(envelop.getUuid());
        List<UserBase> existUsers = userBaseMapper.selectByExample(example);
        if (!existUsers.isEmpty()) {
            log.info("update record for vid: {}", data.vid);
            UserBase existUser = existUsers.get(0);
            userBase.setId(existUser.getId());
            userBase.setCreateTime(existUser.getCreateTime());
            userBaseMapper.updateByPrimaryKeySelective(userBase);
        } else {
            log.info("Inserted new record for vid: {}", data.vid);
            userBaseMapper.insertSelective(userBase);
        }
        // 3. 插入新记录
    }

    public void cancelScanCode(String originalJson) {
        // 1. 解析外层数据
        QwCallBackWrapperData wrapperData = JSON.parseObject(originalJson, QwCallBackWrapperData.class);
        if (StringUtils.isEmpty(wrapperData.getJson())) {
            log.warn("Empty inner json data, uuid: {}", wrapperData.getUuid());
            return;
        }
        // 2. 解析内层用户数据
        String uuid = wrapperData.getUuid();
        //查找库并改写登录状态
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andUuidEqualTo(uuid);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        UserBase user = users.isEmpty() ? null : users.get(0);
        log.info("needVerifyCode, uuid: {}, user: {}", uuid, user);
        if (user != null) {
            user.setLoginStatus(Constant.LOGIN_STATUS_NO_LOGIN);
            userBaseMapper.updateByPrimaryKeySelective(user);
        }
    }

    public void needVerifyCode(String originalJson) {
        // 1. 解析外层数据
        QwCallBackWrapperData wrapperData = JSON.parseObject(originalJson, QwCallBackWrapperData.class);
        if (StringUtils.isEmpty(wrapperData.getJson())) {
            log.warn("Empty inner json data, uuid: {}", wrapperData.getUuid());
            return;
        }
        // 2. 解析内层用户数据
        String uuid = wrapperData.getUuid();
        //查找库并改写登录状态
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andUuidEqualTo(uuid);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        UserBase user = users.isEmpty() ? null : users.get(0);
        log.info("needVerifyCode, uuid: {}, user: {}", uuid, user);
        if (user != null) {
            user.setLoginStatus(Constant.LOGIN_STATUS_LOGIN_VERIFY);
            userBaseMapper.updateByPrimaryKeySelective(user);
        }
    }

    public void handleLogoutScene(String originalJson, int status) {
        QwCallBackWrapperData wrapperData = JSON.parseObject(originalJson, QwCallBackWrapperData.class);
        if (StringUtils.isEmpty(wrapperData.getJson())) {
            log.warn("Empty inner json data, uuid: {}", wrapperData.getUuid());
            return;
        }
        // 2. 解析内层用户数据
        String uuid = wrapperData.getUuid();
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andUuidEqualTo(uuid);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        UserBase user = users.isEmpty() ? null : users.get(0);
        if (user != null) {
            user.setLoginStatus(status);
            userBaseMapper.updateByPrimaryKeySelective(user);
        }
    }

    public void seconderyVerify(String originalJson) {
//        UserBaseExample example = new UserBaseExample();
//        example.createCriteria().andUuidEqualTo(uuid);
//        List<UserBase> users = userBaseMapper.selectByExample(example);
//        UserBase user = users.isEmpty() ? null : users.get(0);
//        if (user != null) {
//            user.setLoginStatus(Constant.LOGIN_STATUS_LOGIN_VERIFY);
//            userBaseMapper.updateByPrimaryKeySelective(user);
//        }
    }

    // 根据手机号查询用户
    public UserBase getUserByMobile(String mobile) {
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andMobileEqualTo(mobile);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        return users.isEmpty() ? null : users.get(0);
    }

    // 分页查询用户列表
    public List<UserBase> listUsers(int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        return userBaseMapper.selectByExample(new UserBaseExample());
    }

    //根据uuid查询用户登录状态
    public UserBase getUserByUuid(String uuid) {
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andUuidEqualTo(uuid);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        return users.isEmpty() ? null : users.get(0);
    }
    //根据名称查询用户
    public List<UserBase> getUserByNickName(String nickname) {
        UserBaseExample example = new UserBaseExample();
        example.createCriteria().andNicknameEqualTo(nickname);
        List<UserBase> users = userBaseMapper.selectByExample(example);
        return users.isEmpty() ? null : users;
    }
}